/****************************************************************************
				Program INtegrity CHecker
				for NetWare 3.x
				Copyright 1992, Novell, Inc.

				Morgan B. Adair
				Novell Systems Research

				CRC.C			Functions to calculate 16-bit cyclical redundancy
								check (CRC) value
*****************************************************************************/

#include <io.h>
#include <fcntl.h>
#include "scanfile.h"
#include "crc.h"

#define TABLE_SIZE	256
#define BUF_SIZE		10240			/* calc CRC of file in 10K hunks */

void CRCBuffer(char *buf, unsigned count, unsigned *crc);

static unsigned crcTable[TABLE_SIZE];
static char buf[BUF_SIZE];
char textBuffer[128];

unsigned CalculateCRC(char *path)
{
	int		handle;
	int		bytesRead;
	unsigned crc = 0;

	if ((handle=open(path, O_BINARY | O_RDONLY)) == -1) {
		sprintf(textBuffer, "Unable to open %s\n", path);
		LogMessage(textBuffer);
	} else {
		do {
			if ((bytesRead=read(handle, buf, BUF_SIZE)) == -1)
				break;
			CRCBuffer(buf, bytesRead, &crc);
			ThreadSwitch();
		} while (bytesRead == BUF_SIZE);

		close(handle);
	}
	return(crc);
}

void InitializeCRCTable(void)
{
	unsigned	i;
	unsigned entry;
	int		index;
	int		carry;

	for (index = 0; index < TABLE_SIZE; ++index) {
		entry = index;

		for (i = 0; i < 8; ++i) {
      	carry = entry & 1;  		/* bit 0 set? */
			entry >>= 1;

			if (carry)
				entry ^= 0xa001;
		}

		crcTable[index] = entry;
	}
}

void CRCBuffer(char *buf, unsigned count, unsigned *crc)
{
	unsigned			i;
	unsigned char	index;	/* 8-bit value to restrict range */

	for (i = 0; i < count; ++i) {
		index = buf[i] ^ *crc;
		*crc >>= 8;
		*crc ^= crcTable[index];
	}
}

