/****************************************************************************
				Program INtegrity CHecker
				for NetWare 3.x
				Copyright 1992, Novell, Inc.

				Morgan B. Adair
				Novell Systems Research

				PINCH.C		Contains main thread and supporting functions
*****************************************************************************/

#include <stdio.h>
#include <conio.h>
#include <time.h>
#include <nwtypes.h>
#include <process.h>
#include "btrieve.h"
#include "pinch.h"
#include "scanfile.h"
#include "datafile.h"

void CleanUp(void);
short GetCommand(void);
void CheckForESC(void);
extern void ClearScreenRow(int row);
extern void UpdateScreen(void);

short	programStatus = STOPPED;
short	runMode = UNDEFINED;
BYTE	threadCount = 0;
int	mainScreen;			/* Handle for main screen (created by main) */
time_t	wakeUpTime;	 	/* Time to begin checking files */
time_t	currentTime;

void main (void)
{
	short	result = BT_SUCCESS;

	threadCount++;

	/* Identify function to call at unload or exit */
	atexit(CleanUp);

	mainScreen = CreateScreen("PINCH Main Screen", 0);
	DisplayScreen(mainScreen);
	HideInputCursor();

	/* Check for Btrieve data file */
	if (!DataFileExists())
		result = CreateDataFile();

	/* If the file could not be created, then exit */
	if (result != 0) {
		programStatus = EXITING;
		DisplayScreen(mainScreen);
		printf("Unable to create data file, Btrieve status = %d\n", result);
	} else
		BeginThread(UpdateScreen, NULL, 0, NULL);

	while (programStatus != EXITING) {

		while (programStatus == STOPPED)
			/* Get command from the user */
   	  	runMode = GetCommand();

		while ((programStatus == RUNNING) ||
				 (programStatus == WAITING)) {

			/* Check to see if it's time to run */
			if (programStatus == WAITING)

				if ((runMode == NOW) ||
					 (runMode == CONSTANT) ||
					 (((runMode == HOURLY) || (runMode == DAILY)) && 
				   	(currentTime >= wakeUpTime))) {

					result = OpenFile();

					if (result == BT_SUCCESS) {
						programStatus = RUNNING;
						ScanFiles();
						result = CloseDataFile();

						if (result != BT_SUCCESS) {
							DisplayScreen(mainScreen);
							printf("Unable to close data file, Btrieve status = %d\n", result);
						}

					} else {
						DisplayScreen(mainScreen);
						printf("Unable to open data file, Btrieve status = %d\n", result);
						programStatus = STOPPED;
					}
				}

			CheckForESC();

			delay(500);	/* catch 500 winks (1/2 second) */

		} /* while programStatus == RUNNING || WAITING */

	} /* while programStatus != EXITING */
	threadCount--;
	return;
}

void CleanUp(void)
{
	programStatus = EXITING;
	while (threadCount > 1)
		ThreadSwitch();

	return;
}

short GetCommand(void)
{
	int			ch;

	if (kbhit())
		return(runMode);
	else {
		ch = getch();
		switch (ch) {
		 		case	0		:	ch = getch();
									return(runMode);

				case	'1'	:	programStatus = WAITING;
									ScheduleRun(NOW);
									return(NOW);

				case	'2'	:	programStatus = WAITING;
									ScheduleRun(HOURLY);
									return(HOURLY);

				case	'3'	:	programStatus = WAITING;
									ScheduleRun(DAILY);
									return(DAILY);

				case	'4'	:	programStatus = WAITING;
									ScheduleRun(CONSTANT);
									return(CONSTANT);

				case ESC		:	programStatus = EXITING;
									return(UNDEFINED);
		}
	}
}

void ScheduleRun(short mode)
{
	struct tm	*timeStruct;

	switch (mode) {
				case	CONSTANT	:
	 			case	NOW		:	wakeUpTime = currentTime;
										break;

				case	HOURLY	:	timeStruct = localtime(&currentTime);
										timeStruct->tm_sec = 0;
										timeStruct->tm_min = 0;
										timeStruct->tm_hour++;
										wakeUpTime = mktime(timeStruct);
										break;

				case	DAILY		:	timeStruct = localtime(&currentTime);
										timeStruct->tm_sec = 0;
										timeStruct->tm_min = 0;
										timeStruct->tm_hour = 0;
										timeStruct->tm_mday++;
										wakeUpTime = mktime(timeStruct);
										break;
	}
}

void CheckForESC(void)
{
	char	ch;

	/* Check for ESC before we go through the loop again */
	while (kbhit()) {
		ch = getch();
		if (ch == ESC) {
			EnterCritSec();
			DisplayScreen(mainScreen);

			gotoxy(25, 20);
			printf("Exit Program Integrity Checker?");
			ExitCritSec();
			do {
				if (kbhit())
					ch = getch();
				ThreadSwitch();
			} while ((toupper(ch) != 'Y') && (toupper(ch) != 'N'));
			if (toupper(ch) == 'Y')
				programStatus = EXITING;
			else {
				EnterCritSec();
				SetCurrentScreen(mainScreen);
				gotoxy(25, 20);
				ClearScreenRow(20);
				ExitCritSec();
			}
		} /* if ch == ESC */
	} /* while kbhit() */
}

