/****************************************************************************
				Program INtegrity CHecker
				for NetWare 3.x
				Copyright 1992, Novell, Inc.

				Morgan B. Adair
				Novell Systems Research

				SCANFILE.C		Scan files on one volume for changes
*****************************************************************************/

#include <conio.h>
#include <direct.h>
#include <stdlib.h>
#include <stdio.h>
#include <nwdir.h>
#include <errno.h>
#include <string.h>
#include <time.h>
#include "btrieve.h"
#include "pinch.h"
#include "scanfile.h"
#include "datafile.h"
#include "crc.h"
#include "log.h"

void PushNode(char *path, DIR *dirStruct);
void PopNode(char *path);
void DecodeDateAndTime(char *time, char *timeString);
extern void CheckForESC(void);

extern short programStatus;
extern short runMode;

SUBDIR_NODE	*subdirStack = NULL;

void ScanFiles(void)
{
	LONG			numberOfVolumes;
	int			ccode;
	int			volumeNumber;
	char			volumeName[_MAX_VOLUME];
	char			dirPath[KEY0_LEN];
	char			searchPath[KEY0_LEN+4];		/* allow for max path plus \*.*   */
	char			filePath[KEY0_LEN];
	SUBDIR_NODE	*tempNode;
	DIR			*directory =  NULL;
	DIR			*directoryEntry;
	char			drive[_MAX_DRIVE];
	char			dir[_MAX_DIR];
	char			fname[_MAX_FNAME];
	char			ext[_MAX_EXT];
	FILE_ENTRY	entry;
	short			result;
	unsigned 	newCRC;
	char			buf[128];
	time_t		currentTime;

	OpenLog();
	LogMessage("\nProgram INtegrity CHecker file scan log\n");
	numberOfVolumes = GetNumberOfVolumes();

	InitializeCRCTable();

	/* for each volume on the file server */
	for (volumeNumber=0; volumeNumber<numberOfVolumes; volumeNumber++) {
		ccode = GetVolumeName(volumeNumber, volumeName);

		if (ccode != ESUCCESS) {
			sprintf(buf, "Unable to get name of volume %d\n", volumeNumber);
			LogMessage(buf);

		} else {
			/* for each volume on the file server */

			currentTime = time(NULL);
			sprintf(buf, "\nBeginning scan of volume %s, %s", volumeName, ctime(&currentTime));
			LogMessage(buf);

			/* create search path for vol's root directory
			   by adding :\*.* to volume name */
			sprintf(searchPath, "%s:\\*.*", volumeName);

			/* scan all files in directory */
			directory = opendir(searchPath);

			/* save copy of path without the :\*.*    */
			sprintf(dirPath, "%s:", volumeName);

			if (!directory) {
				sprintf(buf, "Unable to open volume %s\n", volumeName);
				LogMessage(buf);
			} else {

				do {		/* for each subdirectory on the volume */

					do {	/* while finding file matches */
						directoryEntry = readdir(directory);

						ThreadSwitch();

						if (directoryEntry) {

							/* if directory entry is a subdir,
							   push its name on the subdir stack */
							if (directoryEntry->d_attr & _A_SUBDIR)
								PushNode(dirPath, directoryEntry);
							else {
								/* otherwise, see if it's executable */
							 	_splitpath(directoryEntry->d_name, drive, dir, fname, ext);
								if ((!strcmp (ext, ".EXE")) ||
									 (!strcmp (ext, ".COM")) ||
									 (!strcmp (ext, ".SYS")) ||
									 (!strcmp (ext, ".OVL")) ||
									 (!strcmp (ext, ".BIN")) ||
									 (!strcmp (ext, ".NLM")) ||
									 (!strcmp (ext, ".DSK")) ||
									 (!strcmp (ext, ".LAN")) ||
									 (!strcmp (ext, ".NAM"))) {

								 	sprintf(filePath, "%s\\%s", dirPath, directoryEntry->d_name);
									newCRC = CalculateCRC(filePath);

									result = SearchItem(0, strlen(filePath), (void *)filePath, &entry);

									ThreadSwitch();

									if (result == BT_KEY_VALUE_NOT_FOUND) {
										sprintf(buf, "Inserting record for new file: %s\n", filePath);
										LogMessage(buf);
										strcpy(entry.pathName, filePath);
										entry.checked = TRUE;
										entry.modificationTime = directoryEntry->d_time;
										entry.modificationDate = directoryEntry->d_date;
										entry.fileSize = directoryEntry->d_size;
										entry.lastModifierID = directoryEntry->d_updatedID;
										entry.crc = newCRC;
										ThreadSwitch();
										result = InsertItem(&entry);
										if (result != BT_SUCCESS) {
											sprintf(buf, "Unable to insert record %s, result = %d\n", filePath, result);
											LogMessage(buf);
										}
									} else if (result == BT_SUCCESS) {

										if ((directoryEntry->d_time != entry.modificationTime) ||
											 (directoryEntry->d_date != entry.modificationDate) ||
											 (directoryEntry->d_size != entry.fileSize) ||
											 (directoryEntry->d_updatedID != entry.lastModifierID) ||
											 (entry.crc != newCRC)) {
												LogFileChange(directoryEntry, &entry, newCRC);

												/* update record with new data */
												entry.modificationTime = directoryEntry->d_time;
												entry.modificationDate = directoryEntry->d_date;
												entry.fileSize = directoryEntry->d_size;
												entry.lastModifierID = directoryEntry->d_updatedID;
												entry.crc = newCRC;
										}

										ThreadSwitch();
										entry.checked = TRUE;

										/* Must always update the database entry, even if data has not
										   changed, so we can scan the database later and find files
											that have been deleted */
										result = UpdateItem(&entry);
										if (result != BT_SUCCESS) {
											sprintf(buf, "Unable to update record %s, result = %d\n", filePath, result);
											LogMessage(buf);
										}

									} else {
									 	sprintf(buf, "record not found, result = %d\n", result);
										LogMessage(buf);
									}

									CheckForESC();
								}
							}
						}
					} while ((programStatus != EXITING) &&
						      (directoryEntry));

					closedir(directory);

					/* see if there's another directory entry on stack */
	 				if (subdirStack) {
						PopNode(dirPath);
						sprintf(searchPath, "%s\\*.*", dirPath);
						directory = opendir(searchPath);
						directoryEntry = readdir(directory);
					}

				} while ((programStatus != EXITING) &&
				         (subdirStack) ||
							(directoryEntry));

				/* free any nodes on stack in case of we were
				   Unloaded before run was complete */
				while (subdirStack)
					PopNode(dirPath);

				/* close directory if one was open when Unloaded */
				if (directory)
					closedir(directory);

			} /* if directory else */
		} /* if ccode != ESUCCESS else */
	} /* for each volume */

	DeleteRecordsForDeletedFiles();

	/* Reschedule for next run */
	if (runMode == NOW) {
		programStatus = STOPPED;
		runMode = UNDEFINED;
	} else if (programStatus != EXITING) {
		programStatus = WAITING;
		ScheduleRun(runMode);
	}

	CloseLog();
	return;
}

void PushNode(char *path, DIR *dirStruct)
{
	SUBDIR_NODE	*tempNode;

	tempNode = (SUBDIR_NODE *)malloc(sizeof(SUBDIR_NODE));
	sprintf(tempNode->path, "%s\\%s", path, dirStruct->d_name);
	tempNode->nextNode = subdirStack;
	subdirStack = tempNode;
}

void PopNode(char *path)
{
	SUBDIR_NODE	*tempNode;

	tempNode = subdirStack;
	subdirStack = subdirStack->nextNode;
	strcpy(path, tempNode->path);
	free(tempNode);
}

void DecodeDateAndTime(char *time, char *timeString)
{
	int year, month, day, hour, minute, second,
	    monthMask1 = 0x01,
	    monthMask2 = 0xE0,
	    dayMask = 0x1F,
	    minMask1 = 0x07,
	    minMask2 = 0xE0,
	    secMask = 0x1F;
	
	year = time[1] >> 1;
	year = year + 80;
	month = (time[1] & monthMask1) << 3;
	month = ((time[0] & monthMask2) >> 5) + month;
	day = time[0] & dayMask;

	hour = time[3] >> 3;
	minute = (time[3] & minMask1) << 3;
	minute = ((time[2] & minMask2) >> 5) + minute;
	second = (time[2] & secMask) * 2;

	sprintf(timeString, "%02d-%02d-%02d %02d:%02d:%02d",
		month, day, year, hour, minute, second);
}

